use std::process::{Command, Stdio};
use std::sync::Arc;

use log::{error, trace, warn};
use serde::Deserialize;
use tokio::sync::broadcast;

use crate::modes::key::KeyEvent;

#[derive(Debug, Deserialize)]
pub struct Config {
    pub command: String,
}

pub async fn handle(config: Arc<Config>, mut events: broadcast::Receiver<KeyEvent>) {
    while let Ok(event) = events.recv().await {
        if let KeyEvent::Press = event {
            let result = Command::new("sh")
                .args(["-c", &config.command])
                .stdout(Stdio::null())
                .stderr(Stdio::null())
                .stdin(Stdio::null())
                .status();

            match result {
                Ok(status) => {
                    if status.success() {
                        trace!("Command `{}` exited with status code 0", config.command);
                    } else {
                        warn!("Command `{}` exited with status code {}", config.command, status);
                    }
                }
                Err(error) => error!("Command `{}` could not be executed: {}", config.command, error),
            }
        }
    }
}
