use std::sync::Arc;

use serde::Deserialize;
use tokio::sync::broadcast;

use loupedeck_serial::commands::VibrationPattern;

use crate::modes::key::{KeyEvent, KeyTouchEventKind};
use crate::runner::command::IoWorkerCommand;

#[derive(Debug, Deserialize)]
pub struct Config {
    pub pattern: VibrationPattern,
}

pub async fn handle(config: Arc<Config>, mut events: broadcast::Receiver<KeyEvent>, commands: flume::Sender<IoWorkerCommand>) {
    while let Ok(event) = events.recv().await {
        if let KeyEvent::Touch { kind, .. } = event {
            if kind == KeyTouchEventKind::Start {
                commands.send(IoWorkerCommand::Vibrate { pattern: config.pattern }).unwrap();
            }
        }
    }
}
