use std::sync::Arc;

use serde::{Deserialize, Serialize};
use tokio::sync::broadcast;

use crate::model;
use crate::model::position::KnobPath;
use crate::runner::command::IoWorkerCommand;

pub mod audio_volume;

#[derive(Debug, Eq, PartialEq, Copy, Clone, Serialize, Deserialize)]
pub enum RotationDirection {
    Clockwise,
    Counterclockwise,
}

#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub enum KnobEvent {
    Press,
    ButtonDown,
    ButtonUp,
    Rotate { direction: RotationDirection },
    VisibilityChange { is_visible: bool },
}

pub fn start_handlers(
    knobs: impl Iterator<Item = (KnobPath, Arc<model::knob_page::Knob>)>,
    events: broadcast::Sender<(KnobPath, KnobEvent)>,
    commands: flume::Sender<IoWorkerCommand>,
) {
    for (path, config) in knobs {
        if let Some(c) = &config.mode.audio_volume {
            tokio::spawn(audio_volume::handle(path.clone(), Arc::clone(c), events.subscribe(), commands.clone()));
        }
    }
}
